/* ***** BEGIN LICENSE BLOCK *****
 * Copyright (c) 2010 Joshua Cranmer
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 * ***** END LICENSE BLOCK ***** */
function categoryItem(name) {
  this._name = name;
  this._children = [];
  this.open = false;
}
categoryItem.prototype = {
  get id() { return this._name; },
  get text() { return this._name ? this._name : "Uncategorized"; },
  get level() { return 0; },
  get children() { return this._children; },
  getProperties: function (props) {
    props.AppendElement(categoryMode.$("isServer-true"));
    props.AppendElement(categoryMode.$("serverType-folderCategory"));
    let hasNew = false, subHaveUnread = false;
    for each (let child in this._children) {
      if (child._folder.hasNewMessages)
        hasNew = true;
      if (child._folder.getNumUnread(true) > 0)
        subHaveUnread = true;
    }
    if (hasNew)
      props.AppendElement(categoryMode.$("newMessages-true"));
    if (subHaveUnread)
      props.AppendElement(categoryMode.$("subfoldersHaveUnreadMessages-true"));
  },
  command: function () {},
  addFolder: function (folder) {
    let item = new ftvItem(folder);
    item._level++;
    item.__defineGetter__("children", function() {
      if (!this._children) {
        let iter = fixIterator(this._folder.subFolders,
          Components.interfaces.nsIMsgFolder);
        this._children = [new ftvItem(f) for each (f in iter)];
        sortFolderItems(this._children);
        for each (let child in this._children) {
          child._level = this._level + 1;
          child._parent = this;
        }
        let category = categoryMode._getFolderCategory(this._folder);
        this._children = this._children.filter(function (val) {
          let scat = categoryMode._getFolderCategory(val._folder);
          return scat == "" || scat == category;
        });
      }
      return this._children;
    });
    for each (x in this._children) {
      if (x._folder.abbreviatedName == folder.abbreviatedName) {
        x.addServerName = true;
        item.addServerName = true;
        break;
      }
    }
    this._children.push(item);
  }
};

let categoryMode = {
  _getFolderCategory: function categoryView_getCat(folder) {
    if (folder) {
      let cat = folder.getStringProperty("FolderCategory");
      return cat == null ? "" : cat;
    }
    return "";
  },
  _catItems: {},
  generateMap: function categoryView_generateMap(ftv) {
    this._catItems = {};
    MailUtils.discoverFolders();
    for each (let folder in ftv._enumerateFolders) {
      let category = this._getFolderCategory(folder);
      let use = folder.parent == null ||
        (category && this._getFolderCategory(folder.parent) != category);
      if (use) {
        if (!(category in this._catItems))
          this._catItems[category] = new categoryItem(category);
        this._catItems[category].addFolder(folder);
      }
    }
    let map = [];
    for each (let item in this._catItems)
      map.push(item);
    return map;
  },
  getParentOfFolder: function categoryView_getParent(folder) {
    if (folder instanceof Components.interfaces.nsIMsgFolder)
    {
      return this._catItems[this._getFolderCategory(folder)];
    }
    return null;
  },
  onFolderAdded: function categoryView_onFolderAdded(parent, folder) {},
  $: function categoryView_getAtom(name) {
   return Components.classes["@mozilla.org/atom-service;1"]
                    .getService(Components.interfaces.nsIAtomService)
                    .getAtom(name);
  }
};

categoryMode.__proto__ = IFolderTreeMode;
gFolderTreeView.registerFolderTreeMode("foldercat", categoryMode,
  "Categorized Folders");


function CategorizeFolder() {
  let folder = GetFirstSelectedMsgFolder();
  let categories = [n.id for
    each (n in categoryMode.generateMap(gFolderTreeView))];
  categories.sort();
  if (!categories[0])
    categories.shift();
  window.openDialog("chrome://foldercat/content/foldercat.xul", "props",
    "chrome,modal", folder, categories);
  if (gFolderTreeView.mode == "foldercat")
    gFolderTreeView._rebuild();
}

// XXX: So it turns out that gFolderTreeView.load is broken
/*gFolderTreeView.load = function gftvload_patch(aTree, aJSONFile) {
    const Cc = Components.classes;
    const Ci = Components.interfaces;
    this._treeElement = aTree;

    if ("registerMode" in this) {
      let smartName = document.getElementById("bundle_messenger")
                              .getString("folderPaneHeader_smart");
      this.registerMode("smart", this._smartFoldersGenerator, smartName);
      this._mapItemAdded["smart"] = this._smartItemAdded;
    }

    // the folder pane can be used for other trees which may not have these elements.
    if (document.getElementById("folderpane_splitter"))
      document.getElementById("folderpane_splitter").collapsed = false;
    if (document.getElementById("folderPaneBox"))
      document.getElementById("folderPaneBox").collapsed = false;

    try {
      // Normally our tree takes care of keeping the last selected by itself.
      // However older versions of TB stored this in a preference, which we need
      // to migrate
      let prefB = Cc["@mozilla.org/preferences-service;1"]
                     .getService(Ci.nsIPrefBranch);
      let modeIndex = prefB.getIntPref("mail.ui.folderpane.view");
      // upgrade default All to Smart
      this._mode = (modeIndex == 0) ? kDefaultMode : this._modeNames[modeIndex];
      prefB.deleteBranch("mail.ui.folderpane");
    } catch(ex) {
      // This is ok.  If we've already migrated we'll end up here
    }

    if (document.getElementById('folderpane-title')) {
      let string;
        if (this.mode in this._modeDisplayNames)
          string = this._modeDisplayNames[this._mode];
        else {
          let key;
          if ("registerMode" in this)
            key = "folderPaneHeader_" + this.mode;
          else
            key = "folderPaneModeHeader_" + this.mode;
          string = document.getElementById("bundle_messenger").getString(key);
        }
      document.getElementById('folderpane-title').value = string;
    }

    if (aJSONFile) {
      // Parse our persistent-open-state json file
      let file = Cc["@mozilla.org/file/directory_service;1"]
                    .getService(Ci.nsIProperties).get("ProfD", Ci.nsIFile);
      file.append(aJSONFile);

      if (file.exists()) {
        let data = "";
        let fstream = Cc["@mozilla.org/network/file-input-stream;1"]
                         .createInstance(Ci.nsIFileInputStream);
        let sstream = Cc["@mozilla.org/scriptableinputstream;1"]
                         .createInstance(Ci.nsIScriptableInputStream);
        fstream.init(file, -1, 0, 0);
        sstream.init(fstream);

        while (sstream.available())
          data += sstream.read(4096);

        sstream.close();
        fstream.close();
        let JSON = Cc["@mozilla.org/dom/json;1"].createInstance(Ci.nsIJSON);
        this._persistOpenMap = JSON.decode(data);
      }
    }

    // Load our data
    this._rebuild();
    // And actually draw the tree
    aTree.view = this;

    // Add this listener so that we can update the tree when things change
    let session = Cc["@mozilla.org/messenger/services/session;1"]
                     .getService(Ci.nsIMsgMailSession);
    session.AddFolderListener(this, Ci.nsIFolderListener.all);
  };*/
